@@ -752,7 +752,7 @@ gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
     if (send && have_data) {
       outbuf =
           gst_buffer_new_wrapped_full (GST_MEMORY_FLAG_READONLY, pkt->data,
-          pkt->size, 0, pkt->size, pkt->data, gst_ffmpegvidenc_free_avpacket);
+          pkt->size, 0, pkt->size, pkt, gst_ffmpegvidenc_free_avpacket);
       frame->output_buffer = outbuf;
 
       if (ffmpegenc->context->coded_frame->key_frame)
